#!/usr/bin/python2.7 -B
# -*- coding: UTF-8 -*-
from Tkinter import *
from time import sleep, time
import sys
import os

class Progress:
    def __init__(self, window, description=''):
        window.grid_propagate(True)

        self.frame = Frame(window, borderwidth=2, relief=RAISED)
        self.frame.grid_propagate(True)
        self.frame.grid(sticky="NSEW")
        self.center = lambda *args: self._center(window, self.frame)

        self.title = Label(self.frame, text="University of Helsinki\nUser registration via Eduroam")
        self.title.grid(row=0, column=0, columnspan=3, ipadx=16, ipady=8, sticky="EW")

        self.desc = Label(self.frame, text=description, padx=16, pady=4)
        self.desc.grid(row=1, column=0, columnspan=3, sticky="EW")

        self.progress = Canvas(self.frame, width=255, height=15, borderwidth=2, relief=SUNKEN)
        self.progress.grid(row=2, column=0, columnspan=3, padx=16, pady=8)
        self.marker = self.progress.create_rectangle(0, 0, 64, 16, outline='#99c', fill='#99c')
        self.started = time()

    def spin(self):
        if self.progress is not None:
            elapsed = 0.5 * (time() - self.started)
            phase = 1 - abs(2*(elapsed - int(elapsed)) - 1)
            x = int(192*phase)
            self.progress.delete(self.marker)
            self.marker = self.progress.create_rectangle(x, 0, x+64, 16, outline='#99a', fill='#99a')

    def _center(self, window, frame):
        window.update()
        window.update_idletasks()
        maxw = window.winfo_vrootwidth()
        maxh = window.winfo_vrootheight()
        if frame is None:
            w = min(window.winfo_width(), maxw)
            h = min(window.winfo_height(), maxh)
        else:
            w = min(frame.winfo_width(), maxw)
            h = min(frame.winfo_height(), maxh)
        window.wm_geometry("%dx%d+%d+%d" % (w, h, (maxw-w)/2, (maxh-h)/2))


class Message:
    def __init__(self, window, description=''):
        self.active = True

        window.grid_propagate(True)

        self.frame = Frame(window, borderwidth=2, relief=RAISED)
        self.frame.grid_propagate(True)
        self.frame.grid(sticky="NSEW")
        self.center = lambda *args: self._center(window, self.frame)

        self.title = Label(self.frame, text="University of Helsinki\nUser registration via Eduroam")
        self.title.grid(row=0, column=0, ipadx=16, ipady=8, sticky="EW")

        self.desc = Label(self.frame, text=description, padx=16, pady=4)
        self.desc.grid(row=1, column=0, sticky="EW")

        self.button = Button(self.frame, text="Ok", command=lambda: self.close())
        self.button.bind('<Return>', func=lambda event: self.close())
        self.button.grid(row=2, column=0)

    def focus(self):
        self.button.focus_force()

    def isactive(self):
        return self.active

    def close(self):
        self.active = False

    def _center(self, window, frame):
        window.update()
        window.update_idletasks()
        maxw = window.winfo_vrootwidth()
        maxh = window.winfo_vrootheight()
        if frame is None:
            w = min(window.winfo_width(), maxw)
            h = min(window.winfo_height(), maxh)
        else:
            w = min(frame.winfo_width(), maxw)
            h = min(frame.winfo_height(), maxh)
        window.wm_geometry("%dx%d+%d+%d" % (w, h, (maxw-w)/2, (maxh-h)/2))


if __name__ == '__main__':

    # The path to the marker file is required.
    if (len(sys.argv) != 4) or (len(sys.argv[1]) < 1) or (len(sys.argv[2]) < 1) or (len(sys.argv[3]) < 1):
        sys.stderr.write('\n'
                         'Usage: %s MARKER-PATH PROGRESS-PATH FINAL-PATH\n'
                         '\n' % sys.argv[0])
        sys.exit(1)

    markerpath = sys.argv[1]
    progresspath = sys.argv[2]
    finalpath = sys.argv[3]

    # The marker file must not exist yet.
    if os.path.exists(markerpath):
        sys.stderr.write('%s: Already exists.\n' % markerpath)
        sys.exit(1)

    # Create a fullscreen splash.
    splash = Tk()
    splash.wm_attributes('-zoomed', 'true')
    splash.wm_attributes('-type', 'splash')
    splash.wm_attributes('-topmost', 'false')
    splash.tk_setPalette('#000')

    background = PhotoImage(file='files/background.pnm')

    image = Label(splash, image=background, width=splash.winfo_vrootwidth(), height=splash.winfo_vrootheight())
    image.ref = background
    image.pack()

    # Ensure the splash is mapped and visible.
    splash.update()
    splash.update_idletasks()
    splash.update()

    # Create the marker file.
    with open(markerpath, "w") as handle:
        handle.write("Ok\n")

    # Stay alive for as long as the marker file, and only the marker file) exists.
    while (os.path.exists(markerpath) and
           not os.path.exists(progresspath) and
           not os.path.exists(finalpath)):
        splash.update()
        splash.update_idletasks()
        splash.update()
        sleep(0.02)

    # Try reading the progress spinner description.
    try:
        with open(progresspath, 'r') as handle:
            desc = handle.read()
        desc = desc.strip('\t\n\v\f\r ')
    except:
        desc = None
    if desc is not None:
        # Show progress spinner.
        dialog = Tk()
        dialog.wm_attributes('-zoomed', 'false')
        dialog.wm_attributes('-type', 'splash')
        dialog.wm_attributes('-topmost', 'true')
        dialog.tk_setPalette('#cde')
        progress = Progress(dialog, desc)
        dialog.update()
        dialog.update_idletasks()
        progress.center()
        while (os.path.exists(markerpath) and
               not os.path.exists(finalpath)):
            splash.update()
            splash.update_idletasks()
            dialog.update()
            dialog.update_idletasks()
            progress.spin()
            sleep(0.02)
        dialog.destroy()
        dialog = None

    # Try reading the final message description.
    try:
        with open(finalpath, 'r') as handle:
            desc = handle.read()
        desc = desc.strip('\t\n\v\f\r ')
    except:
        desc = None
    if desc is not None:
        # Show final message.
        dialog = Tk()
        dialog.wm_attributes('-zoomed', 'false')
        dialog.wm_attributes('-type', 'splash')
        dialog.wm_attributes('-topmost', 'true')
        dialog.tk_setPalette('#cde')
        message = Message(dialog, desc)
        dialog.update()
        dialog.update_idletasks()
        message.center()
        message.focus()
        dialog.after(15000, func=message.close)
        while (os.path.exists(markerpath) and message.isactive()):
            splash.update()
            splash.update_idletasks()
            dialog.update()
            dialog.update_idletasks()
            sleep(0.02)
        dialog.destroy()

    splash.destroy()
    sys.exit(0)
