#!/usr/bin/python2.7 -B
# -*- coding: UTF-8 -*-
from Tkinter import *
from time import time, sleep
import sys
import subprocess

class Progress:

    def __init__(self, window):
        self.active = True
        self.canceled = False

        window.grid_propagate(True)

        self.frame = Frame(window, borderwidth=2, relief=RAISED)
        self.frame.grid_propagate(True)
        self.frame.grid(sticky="NSEW")
        self.center = lambda *args: self._center(window, self.frame)

        self.title = Label(self.frame, text="University of Helsinki\nUser registration via Eduroam")
        self.title.grid(row=0, column=0, columnspan=3, ipadx=16, ipady=8, sticky="EW")

        self.desc = Label(self.frame, text=" ", padx=16, pady=4)
        self.desc.grid(row=1, column=0, columnspan=3, sticky="EW")

        self.progress = Canvas(self.frame, width=255, height=15, borderwidth=2, relief=SUNKEN)
        self.progress.grid(row=2, column=0, columnspan=3, padx=16, pady=8)
        self.marker = self.progress.create_rectangle(0, 0, 64, 16, outline='#99c', fill='#99c')
        self.started = time()

        self.button = Button(self.frame, text="Cancel", command=lambda: self.cancel())
        self.button.bind("<Return>", func=lambda event: self.cancel())
        self.button.grid(row=3, column=1)
        Label(self.frame, text=" ").grid(row=3, column=0, ipady=12)
        Label(self.frame, text=" ").grid(row=3, column=2, ipady=12)

    def spin(self):
        if self.progress is not None:
            elapsed = 0.5 * (time() - self.started)
            phase = 1 - abs(2*(elapsed - int(elapsed)) - 1)
            x = int(192*phase)
            self.progress.delete(self.marker)
            self.marker = self.progress.create_rectangle(x, 0, x+64, 16, outline='#99a', fill='#99a')

    def focus(self):
        self.button.focus_force()

    def describe(self, description):
        self.desc.configure(text=description)
        self.center()

    def cancel(self):
        self.active = False
        self.canceled = True

    def fail(self, description, duration=5.0):
        self.desc.configure(text=description)
        if self.progress is not None:
            self.progress.destroy()
            self.progress = None
        self.center()
        self.frame.after(ms=int(1000*duration), func=self.cancel)

    def _center(self, window, frame):
        window.update()
        window.update_idletasks()
        maxw = window.winfo_vrootwidth()
        maxh = window.winfo_vrootheight()
        if frame is None:
            w = min(window.winfo_width(), maxw)
            h = min(window.winfo_height(), maxh)
        else:
            w = min(frame.winfo_width(), maxw)
            h = min(frame.winfo_height(), maxh)
        window.wm_geometry("%dx%d+%d+%d" % (w, h, (maxw-w)/2, (maxh-h)/2))


if __name__ == '__main__':

    if (len(sys.argv) != 3) or (len(sys.argv[1]) < 1) or (len(sys.argv[2]) < 1):
        sys.stderr.write('\n'
                         'Usage: %s CONNECTION-UUID PASSWD-FILE\n'
                         '\n' % sys.argv[0])
        sys.exit(1)

    commands = [

        { 'args': [ 'nmcli', '-w', '15', 'connection', 'up', 'uuid', sys.argv[1], 'passwd-file', sys.argv[2] ],
          'desc': 'Connecting ...',
          'func': lambda: sys.stdin.close(),
          'fail': 'Invalid username or password.',
        },

    ]

    window = Tk()
    window.wm_title(u'University of Helsinki')
    window.wm_attributes('-type', 'splash')
    window.wm_attributes('-topmost', 'true')
    window.tk_setPalette('#cde')

    splash = Progress(window)

    window.update()
    window.update_idletasks()

    splash.center()
    splash.focus()

    for cmd in commands:
        splash.describe(cmd['desc'])
        sub = subprocess.Popen(cmd['args'], bufsize=65536)

        if cmd['func']:
            (cmd['func'])()

        while True:
            splash.spin()

            if sub.poll() is not None:
                break
            if not splash.active:
                break

            window.update()
            window.update_idletasks()

            sleep(0.01)

        if splash.canceled:
            window.destroy()
            sys.exit(1)

        if sub.returncode:
            splash.fail(cmd['fail'])
            while splash.active:
                splash.spin()
                window.update()
                window.update_idletasks()
                sleep(0.01)
            window.destroy()
            sys.exit(1)

    window.destroy()

    if splash.canceled:
        sys.exit(1)
    else:
        sys.exit(0)
